package com.ibm.mqe.model;

/****************************************************************************/
/*                                                                          */
/* Program name: MQ Series Everyplace - Administration Tool                 */
/*                                                                          */
/*  Statement:  Licensed Materials - Property of IBM                        */
/*                                                                          */
/*              MQSeries SupportPac MSE1                                    */
/*              (C) Copyright IBM Corp. 2000                                */
/*                                                                          */
/****************************************************************************/
/*                                                                          */
/* Change history:                                                          */
/*                                                                          */
/* V1.0   15-08-2000  PTvL  Initial release                                 */
/*                                                                          */
/*==========================================================================*/
import  com.ibm.mqe.*;
import  java.util.*;
/**
 *
 */
public class AttributeRule extends MQeRule
  {   /* start class                                                            */
  public    static short  version[] = { 1, 0, 0 };

  protected String cryptors[] = new String[]
								 { "XOR", "DES", "3DES", "RC4", "RC6", "MARS" };

   
  /*----------------------------------------------------------------------------*/
  /*                                                                            */
  /*----------------------------------------------------------------------------*/
  public AttributeRule( )
	{                                           /*                              */
	super( );                                   /* ancestor                     */
	}                                           /*                              */
  /*----------------------------------------------------------------------------*/
  /* compare objects for equal or better                                        */
  /*----------------------------------------------------------------------------*/
  public boolean equals( Object parameter )
	{                                           /*                              */
	int x = mask( parameter );                  /* get the mask value           */
	/* compare authenticators                                                   */
	switch ( x & 0x44 )                         /* just authenticators          */
	  {                                         /*                              */
	  case 0x00:   break;                       /* equal  - none                */
	  case 0x04:   break;                       /* better - Parameter           */
	  case 0x40:   return( false );             /* worse  - Owner               */
	  case 0x44:                                /* equal? - both                */
		{                                       /*                              */
		/* compare authenticators for equality                                  */
		com.ibm.mqe.MQeAttribute Q = (com.ibm.mqe.MQeAttribute) owner;      /* retype owner             */
		com.ibm.mqe.MQeAttribute C = (com.ibm.mqe.MQeAttribute) parameter;  /* retype Parameter         */
		if ( ! Q.getAuthenticator( ).type().equals(C.getAuthenticator( ).type( )) )
		  return( false );                      /* mis-match                    */
		break;                                  /* done                         */
		}                                       /*                              */
	  }                                         /*                              */
	/* compare cryptors                                                         */
	switch ( x & 0x22 )                         /* just cryptors                */
	  {                                         /*                              */
	  case 0x00:   break;                       /* equal  - none                */
	  case 0x02:   break;                       /* better - Parameter           */
	  case 0x20:   return( false );             /* worse  - Owner               */
	  case 0x22:                                /* equal? - both                */
		{                                       /*                              */
		String OwnerSecLevel = ((com.ibm.mqe.MQeAttribute) owner     )
								.getCryptor( ).securityLevel( );
		/* TODO debug only, remove next line                                    */
		trace("i: AttributeRule, equals, OwnerSecLevel = " + OwnerSecLevel );
		String NewSecLevel   = ((com.ibm.mqe.MQeAttribute) parameter )
								.getCryptor( ).securityLevel( );
		/* TODO debug only, remove next line                                    */
		trace("i: AttributeRule, equals, NewSecLevel  = " + NewSecLevel );
		/* find owner seclevel                                                  */
		int i;                                  /* work variable                */
		for ( i = 0; i < cryptors.length; i = i + 1 )   /* find owner           */
		  if ( OwnerSecLevel.equals( cryptors[i] ) )    /* is it this one ?     */
			break;                              /* ... yes                      */
		if ( i > 1 ) i = 2;                     /* set correct scan start point */
		/* is new seclevel equal to or beter than owner ?                       */
		for ( i = i; i < cryptors.length; i = i + 1 )   /* find new             */
		  if ( NewSecLevel.equals( cryptors[i] ) )      /* is it this one       */
			break;                              /* ... yes                      */
		if ( i >= cryptors.length )             /* past end of table ?   >=     */
		  return( false );                      /* ... yes, not equal or better */
		break;                                  /* done                         */
		}                                       /*                              */
	  }                                         /*                              */
	/* compare compresors                                                       */
	switch ( x & 0x11 )                         /* just compressors             */
	  {                                         /*                              */
	  case 0x00:   break;                       /* equal  - none                */
	  case 0x01:   break;                       /* better - Parameter           */
	  case 0x10:   break;                       /* worse  - Owner               */
	  case 0x11:   break;                       /* equal  - both                */
	  }                                         /*                              */
	/* TODO debug only, remove next line                                        */
	trace("i: AttributeRule, equals, returning true");
	return( true );                             /*  return state                */
	}                                           /*                              */
  /*----------------------------------------------------------------------------*/
  /* create the mask                                                            */
  /*----------------------------------------------------------------------------*/
  public int mask( Object two )
	{                                           /*                              */
	int Result = 0;                             /* state mask                   */
	/* Owner - setup of mask                                                    */
	if ( owner instanceof com.ibm.mqe.MQeAttribute )        /* is owner an attribute ?      */
	  {                                         /* ... yes                      */
	  com.ibm.mqe.MQeAttribute A = (com.ibm.mqe.MQeAttribute) owner;    /* retype object                */
	  if ( ! (A.getAuthenticator( ) == null) )  Result = Result + 0x40;
	  if ( ! (A.getCryptor( )       == null) )  Result = Result + 0x20;
	  if ( ! (A.getCompressor( )    == null) )  Result = Result + 0x10;
	  }                                         /*                              */
	/* parameter - setup of mask                                                */
	if ( two instanceof MQeAttribute )          /* is paramter an attribute ?   */
	  {                                         /* ... yes, not attribute       */
	  com.ibm.mqe.MQeAttribute B = (com.ibm.mqe.MQeAttribute) two;      /* retype object                */
	  if ( ! (B.getAuthenticator( ) == null) )  Result = Result + 0x04;
	  if ( ! (B.getCryptor( )       == null) )  Result = Result + 0x02;
	  if ( ! (B.getCompressor( )    == null) )  Result = Result + 0x01;
	  }                                         /*                              */
	/* TODO debug only, remove next line                                        */
	trace("i: AttributeRule, mask value " + byteToHex( intToByte( Result ) ) );
	return( Result );                           /* return mask value            */
	}                                           /*                              */
  /*----------------------------------------------------------------------------*/
  /* Validate new rules                                                         */
  /*----------------------------------------------------------------------------*/
  public MQeRule newRule( Object owner, MQeRule thisRule ) throws Exception
	{                                           /*                              */
	if ( thisRule == null )                     /* Do not allow "no rule"       */
	  return( this );
	return( super.newRule( owner, thisRule ) ); /* check with ancestor          */
	}                                           /*                              */
  /*----------------------------------------------------------------------------*/
  /* process a Permit request                                                   */
  /*----------------------------------------------------------------------------*/
  public Object permit( Object owner, Object from, Object to ) throws Exception
	{                                           /*                              */
	if ( (owner instanceof MQeAttribute) && (to instanceof MQeAttribute) )
	  {                                         /* ... no, not attribute        */
	  int x = mask( to );                       /* state mask                   */
	  /* check Authenticator, Cryptor and compressor states                     */
	  if (  (x & 0x44) == 0x44 )                /* replace authenticator ?      */
		replaceAuthenticator( to );             /* ... yes, is it Ok ?          */
	  if (  (x & 0x22) == 0x22 )                /* replace cryptor ?            */
		replaceCryptor      ( to );             /* ... yes, is it Ok ?          */
	  if (  (x & 0x11) == 0x11 )                /* replace compressor ?         */
		replaceCompressor   ( to );             /* ... yes, is it Ok ?          */
	  /* don't allow removal of Authenticator or cryptor                        */
	  if ( ! (((x & 0x44) == 0x40) || ((x & 0x22) == 0x20)) )   /* is it ?      */
		return( to );                           /* return new object            */
	  }                                         /*                              */
	throw new MQeException( Except_Rule, "Disallowed by rule" );
	}                                           /*                              */
  /*----------------------------------------------------------------------------*/
  /* verify Authenticator replacement                                           */
  /*----------------------------------------------------------------------------*/
  protected void replaceAuthenticator( Object to )
	throws Exception
	{                                           /*                              */
	if ( ((com.ibm.mqe.MQeAttribute)owner).getAuthenticator( ).authenticatedID( ) != null )
	  throw new MQeException ( Except_Rule, "Disallowed by rule" );
	}                                           /*                              */
  /*----------------------------------------------------------------------------*/
  /* verify Compressor replacement                                              */
  /*----------------------------------------------------------------------------*/
  protected void replaceCompressor( Object To )
	throws Exception
	{                                           /*                              */
	}                                           /*                              */
  /*----------------------------------------------------------------------------*/
  /* verify Cryptor replacement                                                 */
  /*----------------------------------------------------------------------------*/
  protected void replaceCryptor( Object to )
	throws Exception
	{                                           /*                              */
	String FromSecLevel = ((com.ibm.mqe.MQeAttribute) owner).getCryptor( ).securityLevel( );
	String ToSecLevel   = ((com.ibm.mqe.MQeAttribute) to   ).getCryptor( ).securityLevel( );
	/* moving to XOR                                                            */
	if ( ToSecLevel.equals( "XOR" ) )
	  for ( int i = 1; i < cryptors.length; i = i + 1 )
		if ( FromSecLevel.equals( cryptors[i] ) )
		  throw new MQeException ( Except_Rule, "Disallowed by rule" );
	/* moving to DES                                                            */
	if ( ToSecLevel.equals( "DES" ) )
	  for ( int i = 2; i < cryptors.length; i = i + 1 )
		if ( FromSecLevel.equals( cryptors[i] ) )
		  throw new MQeException ( Except_Rule, "Disallowed by rule" );
	}                                           /*                              */
}   /* end class                                                             */      
