package com.ibm.mqe.model;

/****************************************************************************/
/*                                                                          */
/* Program name: MQ Series Everyplace - Administration Tool                 */
/*                                                                          */
/*  Statement:  Licensed Materials - Property of IBM                        */
/*                                                                          */
/*              MQSeries SupportPac MSE1                                    */
/*              (C) Copyright IBM Corp. 2000                                */
/*                                                                          */
/****************************************************************************/
/*                                                                          */
/* Change history:                                                          */
/*                                                                          */
/* V1.0   15-08-2000  PTvL  Initial release                                 */
/*                                                                          */
/*==========================================================================*/
import com.ibm.mqe.MQe;
import com.ibm.mqe.MQeFields;
import com.ibm.mqe.MQeAdminMsg;
import com.ibm.mqe.administration.*;
import java.util.*;
import com.ibm.uia.renderingInterface.Reporter;
/**
 * This type implements some utility methods for parsing and
 * checking MQeAdminMessages, especially replies.
 * This class is intended to reman hidden from users, and so
 * is only given package level visibility.
 *
 * @author		Phill van Leersum
 */
public abstract class MQeAdminUtils extends Object {
	private static Reporter reporter;
/**
 * MQeStub.
 * @param name java.lang.String
 */
public MQeAdminUtils() {
	super();
}
/**
 * Add the fields to the given message that indicated
 * that the message requires a reply.  Add a unique
 * (for this machine) identifier to the message, and return
 * the string for matching. 
 * 
 * msg: message to decorate
 * replyQM: where to send reply 
 */
protected String _addReplyRequest(MQeAdminMsg msg, String replyQM) throws Exception {
	return _addReplyRequest(msg, replyQM, Long.toHexString(MQe.uniqueValue()));
}
/**
 * Add the fields to the given message that indicated
 * that the message requires a reply.  Add a unique
 * (for this machine) identifier to the message, and return
 * the string for matching. 
 * 
 * msg: message to decorate
 * replyQM: where to send reply 
 */
protected String _addReplyRequest(MQeAdminMsg msg, String replyQM, String correlate) throws Exception {
	String match = correlate;
	msg.putInt(MQe.Msg_Style, MQe.Msg_Style_Request);
	msg.putAscii(MQe.Msg_ReplyToQ, getReplyToQueueName());
	msg.putAscii(MQe.Msg_ReplyToQMgr, replyQM);
	msg.putArrayOfByte( MQe.Msg_CorrelID, match.getBytes());
	return match;
}
public final void _comment(String s) {
	if (null != reporter) {
		reporter._comment(s);
	}
}
public final void _comment(String s, Exception e) {
	if (null != reporter) {
		reporter._comment(s,e);
	}
}
public final void _error(String s) {
	if (null != reporter) {
		reporter._error(s);
	}
}
public final void _error(String s, Exception e) {
	if (null != reporter) {
		reporter._error(s,e);
	}
}
final String _getReason(MQeAdminMsg replyMsg) {
	try {
		return replyMsg.getReason();
	} catch (Exception e) {
	}
	return "";
}
final int _getReasonCode(MQeAdminMsg replyMsg) {
	return _getReasonCode(_getReason(replyMsg));
}
final int _getReasonCode(String reason) {
	String KEYWORD = "Code=";
	int start = reason.indexOf(KEYWORD);
	if (-1 != start) {
		start += KEYWORD.length();
		int end = reason.indexOf(";", start);
		String str = reason.substring(start, end);
		return Integer.parseInt(str);
	}
	return -1;
}
final int _getResultCode(MQeAdminMsg replyMsg) {
	try {
		return replyMsg.getRC();
	} catch (Exception e) {
	}
	return -1;
}
final boolean _isSuccess(MQeAdminMsg replyMsg) {
	return MQeAdminMsg.RC_Success == _getResultCode(replyMsg);
}
public void _warning(String s) {
	if (null != reporter) {
		reporter._warning(s);
	}
}
public final void _warning(String s, Exception e) {
	if (null != reporter) {
		reporter._warning(s,e);
	}
}
/**
 * Check to see if a good reply was required
 * If not detail the error(s) that occured
 * @return boolean  true if good, or if partly good
 * @param replyMsg reply message to check
 */
public final boolean checkReply(MQeAdminMsg replyMsg) {
	boolean reply = true;
	try {
		// Was a reply received ?
		if (replyMsg == null) {
			//_out().println("..No response received to the request");
			//throw new MQeException("No response message received");
			reply = false;
		} else if (replyMsg.getRC() != MQeAdminMsg.RC_Success) {
			reply = false;
			//_log().println("..Action Failed: " + replyMsg.getReason());
			// If mixed then list each error that occured
			if (replyMsg.getRC() == MQeAdminMsg.RC_Mixed) {
				reply = true;
				MQeFields errors = replyMsg.getErrorFields();
				Enumeration en = errors.fields();
				// process each error
				while (en.hasMoreElements()) {
					String value[];
					String name = (String) en.nextElement();
					value = replyMsg.getFieldInError(name);
					for (int j = 0; j < value.length; j++) {
						//_log().println("Field in error: " + name + "  " + value[j]);
					}
				}
			}
		}
	} catch (Exception e) {
		reply = false;
	}
	return reply;
}
public final String getReplyToQueueName() {
	return MQe.Admin_Reply_Queue_Name;
}
public static final void setReporter(Reporter aReporter) {
	reporter = aReporter;
}
}
