package com.ibm.mqe.model;

/****************************************************************************/
/*                                                                          */
/* Program name: MQ Series Everyplace - Administration Tool                 */
/*                                                                          */
/*  Statement:  Licensed Materials - Property of IBM                        */
/*                                                                          */
/*              MQSeries SupportPac MSE1                                    */
/*              (C) Copyright IBM Corp. 2000                                */
/*                                                                          */
/****************************************************************************/
/*                                                                          */
/* Change history:                                                          */
/*                                                                          */
/* V1.0   15-08-2000  PTvL  Initial release                                 */
/*                                                                          */
/*==========================================================================*/
import com.ibm.uia.metamodel.*;
import com.ibm.uia.xml.*;
import com.ibm.mqe.*;
import java.awt.*;
/**
 * This type was created in VisualAge.
 */
public final class MQeAttribute extends Attribute {
	private static final String TYPE_BOOLEAN = "BOOLEAN";
	private static final String TYPE_INT = "INTEGER";
	private static final String TYPE_LONG = "LONG";
	private static final String TYPE_ASCII = "ASCII";
	private static final String TYPE_UNICODE = "UNICODE";
	private static final String TYPE_STRING = "STRING";
	private static final String TYPE_QUEUE_TARGET_REGISTRY = "REGISTRY_QUEUE_TYPE";
	private static Image synchronisedImage;
	private static Image unsynchronisedImage;	
/**
 * MQAttribute constructor comment.
 * @param def com.ibm.uia.metamodel.AttributeDefinition
 */
public MQeAttribute(AttributeDefinition def, UIObject owner) {
	super(def, owner);
}
/**
 * This method can be broken up polymorphicaly
 * amongst the classes of AttributeType (if we really 
 * want to subclass each).
 */
private final String _getIntegerValue(MQeFields data, String mqeName) {
	String reply = null;
	try {
		int i = data.getInt(mqeName);
		reply = Integer.toString(i);
	} catch (Exception e1) {
		_comment("Attribute: " + getName() + "/" + mqeName + " is not stored as an INTEGER!");
		try {
			byte b = data.getByte(mqeName);
			reply = Byte.toString(b);
			_comment("Attribute: " + getName() + "/" + mqeName + " is stored as a BYTE!");
		} catch (Exception e2) {
			_comment("Attribute: " + getName() + "/" + mqeName + " is not even stored as a BYTE!");
			try {
				long b = data.getLong(mqeName);
				reply = Long.toString(b);
				_comment("Attribute: " + getName() + "/" + mqeName + " is stored as a LONG!");
			} catch (Exception e3) {
				_comment("Attribute: " + getName() + "/" + mqeName + " is not even stored as a LONG!");
			try {
				short b = data.getShort(mqeName);
				reply = Short.toString(b);
				_comment("Attribute: " + getName() + "/" + mqeName + " is stored as a SHORT!");
			} catch (Exception e4) {
				_comment("Attribute: " + getName() + "/" + mqeName + " is not even stored as a SHORT!");
			}
			}
		}
	}
	return reply;
}
/**
 * This method can be broken up polymorphicaly
 * amongst the classes of AttributeType (if we really 
 * want to subclass each).
 */
private final String _readValue(MQeFields data, String defaultValue) {
	String reply = defaultValue;
	String mqeName = getDefinition().getMappedName();
	String strType = getDefinition().getType().getName();
	try {
		if (!data.contains(mqeName)) {
			_comment("Cannot find attribute: " + getName() + "(" +
								mqeName + ")");
		} else {
			int iType = data.dataType(mqeName); 
			switch (iType) {
				case MQeField.TypeAscii:
					if (!TYPE_ASCII.equals(strType)) {
						_warnTypeMismatch(mqeName, MQeFields.Tascii);
					}
					reply = data.getAscii(mqeName);
					break;
				case MQeField.TypeUnicode:
					if (!TYPE_UNICODE.equals(strType)) {
						_warnTypeMismatch(mqeName, MQeFields.Tunicode);
					}
					reply = data.getUnicode(mqeName);
					break;
				case MQeField.TypeBoolean:
					if (!TYPE_BOOLEAN.equals(strType)) {
						_warnTypeMismatch(mqeName, MQeFields.Tboolean);
					}
					reply = data.getBoolean(mqeName) ? "true" : "false";
					break;
				case MQeField.TypeByte:
					byte b = data.getByte(mqeName);
					reply = Byte.toString(b);
					break;
				case MQeField.TypeShort: break;
				case MQeField.TypeInt:
					if (!TYPE_INT.equals(strType)) {
						_warnTypeMismatch(mqeName, MQeFields.Tint);
					}
					int i = data.getInt(mqeName);
					reply = Integer.toString(i);
					break;
				case MQeField.TypeLong:
					if (!TYPE_LONG.equals(strType)) {
						_warnTypeMismatch(mqeName, MQeFields.Tlong);
					}
					long l = data.getLong(mqeName);
					reply = Long.toString(l);
					break;
				case MQeField.TypeFloat: break;
				case MQeField.TypeDouble: break;
				case MQeField.TypeUnTyped: break;
				case MQeField.TypeArrayElements: break;
			}
			reply = MQe.abbreviate(reply, 0);
			reply = getType().unmap(reply);
			//_log().println("Attribute updated:   " + getName());
		}
	} catch (Exception e) {
		_comment("Error reading attribute: " + getName());
		//e.printStackTrace(_log());
	}
	return reply;
}
/**
 */
private final void _warnTypeMismatch(String mqeName, String mqeTypeName) {
	_comment("Attribute " + getName() + "(" + getType().getName()
			+ ") found in mqeField as " + mqeName + "(" + mqeTypeName +")"); 
}
/**
 * This method can be broken up polymorphicaly
 * amongst the classes of AttributeType (if we really 
 * want to subclass each).
 */
final boolean _writeValue(MQeFields data) {
	// note that type STRING will not be read (it is
	// only used for 'our' attributes
	boolean changed = !isCommitted();
	String reply = null;
	String mqeName = getDefinition().getMappedName();
	String type = getDefinition().getType().getName();
	String outValue = getType().getMappedValue(getValue());
	try {
		if (TYPE_LONG.equals(type)) {
			long l = Long.parseLong(outValue);
			data.putLong(mqeName, l);
		} else if (TYPE_INT.equals(type)) {
			int i = Integer.parseInt(outValue);
			data.putInt(mqeName, i);
		} else if (TYPE_ASCII.equals(type)) {
			data.putAscii(mqeName, outValue);
		} else if (TYPE_UNICODE.equals(type)) {
			data.putUnicode(mqeName, outValue);
		} else if (TYPE_BOOLEAN.equals(type)) {
			data.putBoolean(mqeName, outValue == "true" );
		} else if (TYPE_QUEUE_TARGET_REGISTRY.equals(type)) {
			byte b = Byte.parseByte(outValue);
			data.putByte(mqeName, b);
		} else if (getType().isEnumeratedType()) {
			//rest of the enumerated types are ascii
			data.putAscii(mqeName, outValue);
		}
		//_log().println("Attribute written: " + getName());
	} catch (Exception e) {
		_comment("Error writing attribute " + getName() +"(" + e.getMessage() + ")");
	}
	return changed;
}
public java.awt.Image getCaptionImage() {
	java.awt.Image im = null;
	if (isCommitted()) {
		im = getImage("_attrsync.gif");
	} else {
		im = getImage("_attrunsync.gif");
	}
	return im;
}
public java.awt.Color getDisplayColour() {
	if (isCommitted()) {
		return java.awt.Color.black;
	} else {
		return java.awt.Color.blue;
	}
}
private final boolean hasMappedName() {
	String mappedName = getDefinition().getMappedName();
	return null != mappedName && mappedName.length() > 0;
}
/**
 * This method reads the value of the receiver from
 * the given MQeFields object
 */
final void readValueFrom(MQeFields data) {
	if (hasMappedName()) {
		String val = _readValue(data, getValue()); 
		setValue(val);
		setCommittedValue(val);
	}
}
/* setValue .
 * Overridden here because non-mqe attributes
 * are always considered reconciled
 */
public final void setValue(String newValue) {
	super.setValue(newValue);
	if (!hasMappedName()) {
		setCommittedValue(newValue);
	}
}
/**
 * This method writes the value of the attribute into
 * the given fields object, providing: 
 * 	1) the value has changed
 *	2) the attribute is editable.
 *	3) the attribute has a valid mqe TYPE
 *	4) the attribute has an mqeName
 */
final boolean writeValueInto(MQeFields data) {
	boolean change = !isCommitted() && isEditable() && hasMappedName();
	if (change) {
		_writeValue(data);
	}
	return change;
}
}
