package com.ibm.mqe.model;

/****************************************************************************/
/*                                                                          */
/* Program name: MQ Series Everyplace - Administration Tool                 */
/*                                                                          */
/*  Statement:  Licensed Materials - Property of IBM                        */
/*                                                                          */
/*              MQSeries SupportPac MSE1                                    */
/*              (C) Copyright IBM Corp. 2000                                */
/*                                                                          */
/****************************************************************************/
/*                                                                          */
/* Change history:                                                          */
/*                                                                          */
/* V1.0   15-08-2000  PTvL  Initial release                                 */
/*                                                                          */
/*==========================================================================*/
import com.ibm.uia.metamodel.*;
import com.ibm.uia.renderingInterface.*;
import com.ibm.uia.xml.*;
import java.util.*;
import com.ibm.mqe.*;
/**
 * Insert the type's description here.
 * Creation date: (03/07/2000 07:41:24)
 * @author: Phill van Leersum
 */
public class MQeModel extends MQeObject implements Reporter {
	private static final String CMD_ADD_QM = "Administer QueueManager";
	private static final String CMD_SHOW_ADMIN = "Show Administrator";
	private static final String CMD_HIDE_ADMIN = "Hide Administrator";
	
	private Renderer renderer;
	private MQeQueueManager adminQm;
	private QueueManager directQM;
	private boolean showingQMDirect = false;
/**
 * MQMetaModel constructor comment.
 * @param element org.w3c.dom.Element
 */
public MQeModel() {
	super();
}
private boolean _addQM(Renderer renderer, String[] data, int qmMode) {
	if (null != data) {
		for (int i = 0; i < data.length; i++) {
			if (null == data[i]) {
				return false;
			}
		}
		String address = data[1] + ":" + data[2] + ":" + data[3];
		QueueManager admQM = getAdministeredQM(data[0]);
		if (null == admQM) {
			admQM = (QueueManager) addNewChild(MQE_QUEUE_MANAGER_TYPE_NAME, data[0]);
			admQM.setDirect(false, adminQm);
			admQM.setAttribute(QM_ROUTE_ATTRIBUTE, address);
		}
	}
	return true;
}
boolean _addQMDirect(Renderer renderer, String name) {
	if (null != name) {
		directQM = (QueueManager)addNewChild(MQE_QUEUE_MANAGER_TYPE_NAME, name);
		directQM.setDirect(true, adminQm);
		removeChild(directQM);
	}
	return true;
}
/**
 * Read the receivers data from the output data field
 */
protected boolean _readRefreshData(MQeFields data, Vector found) {
	return false;
}
/**
 * If the receiver is connected, then attempt to commit all outstanding
 * changes (creates/deletes/updates).
 */
public boolean commit(boolean recurse) {
	return true;
}
QueueManager getAdministeredQM(String name) {
	QueueManager qm = (QueueManager)findExistingChild(MQE_QUEUE_MANAGER_TYPE_NAME, name);
	return qm;
}
public UIObject getRootObject() {
	return this;
}
boolean hideQMDirect() {
	if (showingQMDirect) {
		removeChild(directQM);
		showingQMDirect = false;
	}
	return true;
}
public boolean isOperationEnabled(RenderableOperation rop) {
	boolean reply = true;
	String command = rop.getName();
	if (CMD_HIDE_ADMIN.equals(command)) {
		reply = true;
	} else if (CMD_SHOW_ADMIN.equals(command)) {
		reply = true;
	}
	return reply;
}
/**
 * Perform the actions represented by the supplied instance of 
 * <code>RenderableOperation</code> defined in te metamodel file.
 * If the operation was defined with a prompt then the String array
 * will be non-null, and will contain a string for every line of the prompt
 *
 * The renderer responsible for delivering the operation request is passed in
 * for convenience.
 *
 * Reply true if a stuctural refresh is required.
 */
public final boolean performOperation(RenderableOperation op, Renderer renderer) {
	String[] data = op.getData();
	String command = op.getName();
	boolean reply = false;
	if (CMD_ADD_QM.equals(command)) {
		reply = _addQM(renderer, data, MQeQueueManagerStub.CONNECTION_MODE_ASYNCHRONOUS);
	} else if (CMD_HIDE_ADMIN.equals(command)) {
		reply = hideQMDirect();
	} else if (CMD_SHOW_ADMIN.equals(command)) {
		reply = showQMDirect();
	} else {
		reply = super.performOperation(op, renderer);
	}
	return reply;
}
/**
 * Remove a representation of the specified queueManager,
 * if one exists.
 *
 * name: name of qm to which the route points
 * routeAddress:  address of machine/port upon which the QM is listening
 */
final boolean removeAdministeredQM(QueueManager admQM) {
	if (null != admQM) {
		admQM.disconnect();
		removeChild(admQM);
		return true;
	}
	return false;
}
/**
 * Remove a representation of the specified queueManager,
 * if one exists.
 *
 * name: name of qm to which the route points
 * routeAddress:  address of machine/port upon which the QM is listening
 */
protected final boolean removeAdministeredQM(String name) {
	return removeAdministeredQM(getAdministeredQM(name));
}
public final void setAdminQM(MQeQueueManager adminQm) {
	this.adminQm = adminQm;
}
boolean showQMDirect() {
	if (!showingQMDirect) {
		addNewChild(directQM);
		showingQMDirect = true;
	}
	return true;
}
}
