package com.ibm.mqe.model;

/****************************************************************************/
/*                                                                          */
/* Program name: MQ Series Everyplace - Administration Tool                 */
/*                                                                          */
/*  Statement:  Licensed Materials - Property of IBM                        */
/*                                                                          */
/*              MQSeries SupportPac MSE1                                    */
/*              (C) Copyright IBM Corp. 2000                                */
/*                                                                          */
/****************************************************************************/
/*                                                                          */
/* Change history:                                                          */
/*                                                                          */
/* V1.0   15-08-2000  PTvL  Initial release                                 */
/*                                                                          */
/*==========================================================================*/
import java.util.*;
import com.ibm.mqe.*;
import com.ibm.mqe.mqbridge.*;

import com.ibm.mqe.administration.*;

/**
 * This type represents an administered MQeQueueManager.
 * Each instance of this type will own and manage an instance of
 * <code>MQeAdminConnection</code> which represents the MQe resources
 * required to pass admin messages to the MQeQueueManager represented.
 * Creation date: (19/05/2000 14:30:21)
 * @author: Phill van Leersum
 */
public abstract class MQeQueueManagerStub extends MQeAdminUtils {
	public static final int CONNECTION_MODE_ASYNCHRONOUS = 2;
	public static final int CONNECTION_MODE_SYNCHRONOUS = 1;
	public static final int CONNECTION_MODE_DISCONNECTED = 0;
	
	/**
	 * Each stub has a name to identify it.  The name must be
	 * unique within its namespace
	 */ 
	private String name;
	/**
	 * Each stub may have a listener, to whom reply messages are sent
	 * when performing asynchonously.
	 */
	private MQeStubMessageListener listener;
	protected MQeQueueManager adminQM;
/**
 * MQeQueueManager constructor.  This is made protected to guide users
 * to the creation method supplied in MQeAdministeringQM.  Honouring
 * the factory pattern will enable developers to extend the capabilities
 * of this code by subclassing (rather than by editing).
 * @param metaObject com.ibm.mq.mmodel.MQeAdministeringQM
 */
public MQeQueueManagerStub(MQeStubMessageListener listener, MQeQueueManager adminQM) {
	super();
	this.listener = listener;
	this.adminQM = adminQM;
	getAdminQueueManagerName();
}
protected final MQeAdminMsg _waitForMessage(MQeQueueManager adminQM, String qmgrName, String match) throws Exception {
	int milliseconds = 10000;
	int GRAIN = 100;
	MQeAdminMsg msg = null;
	MQeFields filter = new MQeFields();
	filter.putArrayOfByte(MQe.Msg_CorrelID, match.getBytes());
	String queueName = getReplyToQueueName();

	while (milliseconds > GRAIN && null == msg) {
		try {
			msg = (MQeAdminMsg)adminQM.getMessage(qmgrName, queueName, filter, null, 0);
		} catch (MQeException ex) {
			if (ex.code() == MQe.Except_Q_NoMatchingMsg) {
			} else {
				throw(ex);
			}
		}
		Thread.sleep(GRAIN);
		milliseconds -= GRAIN;
	}
	return msg;
}

/**
 * boolean[0]: is there a route
 * boolean[1] do e need to delete the route
 */
public abstract boolean[] addTemporaryRoute(String destination, String route);
/**
 * reply an admin message for the MQe resource 
 * represented by the receiver.
 */
private final MQeAdminMsg adminMessage() {
	MQeAdminMsg msg = null;
	try {
		msg = new MQeQueueManagerAdminMsg();
		msg.setName(getName());	// name of resource
	} catch (Exception e) {
		//e.printStackTrace(_log());
	}
	return msg;
}
/**
 * browse messages assciated with the receiver
 */
public abstract MQeEnumeration browseQueue(String queueQueueManagerName, String queueName);
public abstract boolean connect(String name, String route, int mode);
/**
 * reply boolean[0] is there a remote queue
 * reply boolean[1] do we need to delete the queue
 */
public abstract boolean[] createTemporaryRemoteQueue(String queueQueueManagerName, String queueName);
public abstract boolean deleteMessage(String queueQueueManagerName, String queueName, MQeFields msg);
public abstract void deleteTemporaryRemoteQueue(String queueQueueManagerName, String queueName);
public abstract boolean disconnect(boolean removeResources);
/**
 * get reference to Queue Manager performing
 * admin functions
 */
protected final MQeQueueManager getAdminQueueManager() {
	
	return adminQM;
}
/**
 * get reference to Queue Manager performing
 * admin functions
 */
protected final String getAdminQueueManagerName() {
	try {
		return getAdminQueueManager().getName();
	} catch (Exception e) {
		
	}
	return null;
}
/**
 * Reply the <code>MQeStubMessageListener</code> associated with the receiver
 */
protected final MQeStubMessageListener getListener() {
	return listener;
}
public final String getName() {
	return name;
}
/**
 * inquire all fields.
 *
 * If no listener has been set then the method waits for a response to 
 * the admin message, and returns it.
 * If a listener has been set, then the message will be returned
 * asynchronously via the listeners acceptMessage() method.
 */
public final MQeAdminMsg inquireAll() {
	MQeAdminMsg replyMsg = null;
	try {
		MQeAdminMsg msg = adminMessage();
		msg.setAction(MQeAdminMsg.Action_InquireAll);
		replyMsg = putAdminMessage(msg, getListener());
	} catch (Exception e) {
		e.printStackTrace();
	}
	return replyMsg;
}
public abstract boolean isConnected();
/**
 * put an admin message.
 *
 * If no listener has been set then the method waits for a response to 
 * the admin message, and returns it.
 * If a listener has been set, then the message will be returned
 * asynchronously via the listeners acceptMessage() method.
 */
public abstract MQeAdminMsg putAdminMessage(MQeAdminMsg msg, MQeStubMessageListener listener);
public final MQeAdminMsg queryBridgeData() {
	MQeAdminMsg reply = null;
	try {
		MQeAdminMsg msg = new MQeMQBridgesAdminMsg();
		msg.inquireAll(new MQeFields());
		reply = putAdminMessage(msg, getListener());
	} catch (Exception e) {
		_warning("exception while querying bridge data", e);
	}
	return reply;
}
public abstract void removeTemporaryRoute(String destination);
public final void setName(String s) {
	name = s;
}
/**
 * browse messages assciated with the receiver
 */
public abstract void testMessage(String queueQueueManagerName, String queueName, String txt);
/**
 * update all attributes as required.
 * If no listener has been set then the method waits for a response to 
 * the admin message, and returns it.
 * If a listener has been set, then the message will be returned
 * asynchronously via the listeners acceptMessage() method.
 */
public final MQeAdminMsg update(MQeFields data) {
	MQeAdminMsg replyMsg = null;
	try {
		MQeAdminMsg msg = adminMessage();
		//msg.setMaxAttempts(1);
		msg.update(data);
		replyMsg = putAdminMessage(msg, getListener());
	} catch (Exception e) {
		e.printStackTrace();
	}
	return replyMsg;
}
}
