package com.ibm.mqe.model;

/****************************************************************************/
/*                                                                          */
/* Program name: MQ Series Everyplace - Administration Tool                 */
/*                                                                          */
/*  Statement:  Licensed Materials - Property of IBM                        */
/*                                                                          */
/*              MQSeries SupportPac MSE1                                    */
/*              (C) Copyright IBM Corp. 2000                                */
/*                                                                          */
/****************************************************************************/
/*                                                                          */
/* Change history:                                                          */
/*                                                                          */
/* V1.0   15-08-2000  PTvL  Initial release                                 */
/*                                                                          */
/*==========================================================================*/
import java.util.*;
import com.ibm.mqe.*;
import com.ibm.mqe.administration.*;
import com.ibm.uia.renderingInterface.*;
import com.ibm.uia.metamodel.*;
/**
 * This type represents an MQe resource managed by
 * an MQeQeueManager.
 * Creation date: (31/05/2000 10:56:47)
 * @author: Phill van Leersum
 */
abstract class ManagedObject extends CreatableObject{
private final boolean _checkAddAliasReply(MQeAdminMsg replyMsg) {
	boolean reply = false;
	try {
		if (replyMsg == null) {
			reply = false;
		} else
			if (_isSuccess(replyMsg)) {
				MQeFields mqef = replyMsg.getOutputFields();
				String[] alii = getAliasData(mqef);
				for (int i = 0; i < alii.length; i++) {
					CreatableObject mqeo = (CreatableObject) findExistingChild(ALIAS_TYPE_NAME, alii[i]);
					if (null == mqeo) {
						mqeo = (CreatableObject) addNewChild(ALIAS_TYPE_NAME, alii[i]);
					}
					if (null != mqeo) {
						mqeo.setStateCreated();
					}
				}
				reply = true;
			} else {
				MQeFields input = replyMsg.getInputFields();
				String aliasName = getAliasData(input)[0];
				CreatableObject alias = (CreatableObject) findExistingChild(ALIAS_TYPE_NAME, aliasName);
				if (null != alias) {
					alias.setStateToBeCreated();
				}
				String reason;
				if (replyMsg.getRC() == MQeAdminMsg.RC_Mixed) {
					MQeFields errors = replyMsg.getErrorFields();
					reason = getAliasData(errors)[0];
				} else {
					reason = replyMsg.getReason();
				}
				_warning("failed to create alias: " + aliasName + " because: " + reason);
				reply = true;
			}
	} catch (Exception e) {
		_error("add alias reply", e);
	}
	return reply;
}
private final boolean _checkRemoveAliasReply(MQeAdminMsg replyMsg) {
	boolean reply = false;
	try {
		if (replyMsg == null) {
			reply = false;
		} else if (_isSuccess(replyMsg)) {
			MQeFields mqef = replyMsg.getOutputFields();
			String[] alii = getAliasData(mqef);
			for (int i = 0; i < alii.length; i++){
				UIObject uio = findExistingChild(ALIAS_TYPE_NAME, alii[i]);
				if (null != uio) {
					removeChild(uio);
				}
			}
			reply = true;
		} else {
				MQeFields input = replyMsg.getInputFields();
				String aliasName =getAliasData(input)[0];
				CreatableObject alias = (CreatableObject) findExistingChild(ALIAS_TYPE_NAME, aliasName);
				if (null != alias) {
					alias.setStateToBeDeleted();
				}
				String reason;
				if (replyMsg.getRC() == MQeAdminMsg.RC_Mixed) {
					MQeFields errors = replyMsg.getErrorFields();
					reason = getAliasData(errors)[0];
				} else {
					reason = replyMsg.getReason();
				}
			_warning("failed to delete alias: " + aliasName + " because: " + reason);
			reply = true;
		}
	} catch (Exception e) {
		_error("remove alias reply" , e);
	}
	return reply;
}
/**
 * Read the receivers data from the output data field
 */
boolean _readAliasData(MQeFields data, Vector found) {
	boolean changed = false;
	String[] alii = null;
	alii = getAliasData(data);
	if (null != alii) {
		for (int i = 0; i < alii.length; i++) {
			String name = alii[i];
			String type = ALIAS_TYPE_NAME;
			CreatableObject uio = (CreatableObject)findExistingChild(type, name);
			if (null == uio) {
				uio = (CreatableObject)addNewChild(type, name);
				changed = true;
			}
			if (null != uio) {
				found.addElement(uio);
				uio.setStateCreated();
			}
		}
	}
	return changed;
}
/**
 * Read the receivers data from the output data field
 */
protected boolean _readRefreshData(MQeFields data, Vector found) {
	boolean reply = super._readRefreshData(data, found);
	if (isCreating()) {
		refresh();
	}
	_addPendingChildrenTo(found);
	if (_readAliasData(data, found)) {
		reply = true;
	}
	return reply;
}
/**
 * performOperation method comment.
 */
protected boolean _reallyCreateAlias(String aliasName) {
	boolean reply = false;
	if (null != aliasName) {
		MQeAdminMsg replyMsg = addAlias(aliasName);
		reply = handleMessage(replyMsg);
	}
	return reply;
}
/**
 * performOperation method comment.
 */
protected boolean _reallyDeleteAlias(String aliasName) {
	boolean reply = false;
	if (null != aliasName) {
		UIObject uio = findExistingChild(ALIAS_TYPE_NAME, aliasName);
		if (null != uio) {
			MQeAdminMsg replyMsg = removeAlias(aliasName);
			reply = handleMessage(replyMsg);
		}
	}
	return reply;
}
/**
 * performOperation method comment.
 */
public abstract MQeAdminMsg addAlias(String aliasName);
protected abstract MQeAdminMsg adminMessage();
/**
 * commit the receiver and its attributes
 * reply true if any change occurs
 */
public final boolean create() {
	return commit(false);
}
/**
 * performOperation method comment.
 */
protected boolean createAlias(Renderer renderer, String[] data) {
	boolean reply = false;
	String aliasName = data[0];
	if (null != aliasName) {
		if (null == findExistingChild(ALIAS_TYPE_NAME, aliasName)) {
			MQeObject mqeo = (MQeObject)addNewChild(ALIAS_TYPE_NAME, aliasName);
			mqeo.commit(true);
		}
	}
	return true;
}
//	private MQeManagedObjectStub stub;
/**
 * get the alias data, if any, from the MQe fields object
 */
public abstract String[] getAliasData(MQeFields data);
/**
 * reply the name of the queue manager WITHOUT the
 * route definition
 */
final String getRouteAddress() {
	return getAttributeValue(CONNECTION_ROUTE_ATTRIBUTE);
}
/**
 * This is the method called to handle all reply messages, whether delivered
 * asynchronously (through the callback method 'acceptMessage') or synchronously
 * as a returned value from a stub operation.
 */
protected boolean handleMessage(MQeAdminMsg msg) {
	boolean changed = false;
	if (null != msg) {
		switch (msg.getAction()) {
			//case MQeConnectionAdminMsg.Action_AddAlias:
			case MQeQueueAdminMsg.Action_AddAlias :
				changed = _checkAddAliasReply(msg);
				break;
				//case MQeConnectionAdminMsg.Action_RemoveAlias:
			case MQeQueueAdminMsg.Action_RemoveAlias :
				changed = _checkRemoveAliasReply(msg);
				break;
			default :
				changed = super.handleMessage(msg);
				break;
		}
	}
	return changed;
}
/**
 * Perform the actions represented by the supplied instance of 
 * <code>RenderableOperation</code> defined in te metamodel file.
 * If the operation was defined with a prompt then the String array
 * will be non-null, and will contain a string for every line of the prompt
 *
 * The renderer responsible for delivering the operation request is passed in
 * for convenience.
 *
 * Reply true if a stuctural refresh is required.
 */
public boolean performOperation(RenderableOperation op, Renderer renderer) {
	String[] data = op.getData();
	String command = op.getName();
	boolean reply = false;
	if (CMD_ADDALIAS.equals(command)) {
		reply = createAlias(renderer, data);
	} else {
		reply = super.performOperation(op, renderer);
	}
	return reply;
}
public boolean refresh() {
	boolean reply = false;
	if (isConnected()) {
		MQeAdminMsg replyMsg = null;
		try {
			MQeAdminMsg msg = adminMessage();
			msg.setAction(MQeAdminMsg.Action_InquireAll);
			replyMsg = putAdminMessage(msg);
		} catch (Exception e) {
			e.printStackTrace();
		}
		reply = handleMessage(replyMsg);
	}
	return reply;
}
/**
 * performOperation method comment.
 */
public abstract MQeAdminMsg removeAlias(String aliasName);
/**
 * Initiate the sequence of actions which will result in updating the
 * resource represented by the receiver with the values curently held by
 * the receiver.
 * Reply true if this results in a structural change.
 */
protected boolean update(boolean recurse) {
	boolean reply = false;
	MQeFields sendData = new MQeFields();
	_writeAttributes(sendData);

	MQeAdminMsg replyMsg = null;
	try {
		MQeAdminMsg msg = adminMessage();
		//msg.setMaxAttempts(1);
		msg.update(sendData);
		replyMsg = putAdminMessage(msg);
	} catch (Exception e) {
		e.printStackTrace();
	}
	
	reply = handleMessage(replyMsg);
	if (recurse) {
		Enumeration enum = getAllChildren();
		while (enum.hasMoreElements()) {
			MQeObject mqeo = (MQeObject) enum.nextElement();
			if (mqeo.update(recurse)) {
				reply = true;
			}
		}
	}
	return reply;
}
}
