package com.ibm.mqe.model;

/****************************************************************************/
/*                                                                          */
/* Program name: MQ Series Everyplace - Administration Tool                 */
/*                                                                          */
/*  Statement:  Licensed Materials - Property of IBM                        */
/*                                                                          */
/*              MQSeries SupportPac MSE1                                    */
/*              (C) Copyright IBM Corp. 2000                                */
/*                                                                          */
/****************************************************************************/
/*                                                                          */
/* Change history:                                                          */
/*                                                                          */
/* V1.0   15-08-2000  PTvL  Initial release                                 */
/*                                                                          */
/*==========================================================================*/
import java.awt.*;
import java.util.*;
import com.ibm.mqe.*;
import com.ibm.mqe.administration.*;
import com.ibm.uia.metamodel.*;
import com.ibm.uia.renderingInterface.*;
/**
 * This type represents an MQeQueueManager.  This type should be visible 
 * only within this package.
 * NOTE: the 'name' as used by superclasses consists of the queueManager 
 * name plus '@' plus the route.  This gives a unique identity.
 * Creation date: (19/05/2000 14:30:21)
 * @author: Phill van Leersum
 */
public class QueueManager extends MQeObject {
	
	private static final String CMD_UNDO_CHANGES = "Undo Changes";
	private static final String CMD_NEW_CONNECTION = "DirectConnection";
	private static final String CMD_NEW_INDIRECT_CONNECTION = "IndirectConnection";
	private static final String CMD_CONNECT_SYNC = "ConnectSync";
	private static final String CMD_CONNECT_ASYNC = "ConnectAsync";
	private static final String CMD_DISCONNECT = "Disconnect";
	private static final String CMD_NEW_LOCAL_QUEUE = "LocalQueue";
	private static final String CMD_NEW_REMOTE_QUEUE_SYNC = "SyncRemoteQueue";
	private static final String CMD_NEW_REMOTE_QUEUE_ASYNC = "AsyncRemoteQueue";
	private static final String CMD_NEW_STOREANDFORWARD_QUEUE = "StoreAndForwardQueue";
	private static final String CMD_NEW_HOMESERVER_QUEUE = "HomeServerQueue";

	private static final String STATE_DISCONNECTED = "Disconnected";
	private static final String STATE_CONNECTING = "Connecting";
	private static final String STATE_CONNECTED = "Connected";
	private static final String STATE_DISCONNECTING = "Disconnecting";
	
	private static final String[][] classNameToTypeMap = {
		{"com.ibm.mqe.MQeQueue", 					LOCALQUEUE_TYPE_NAME},
		{"com.ibm.mqe.MQeAdminQueue",		 		ADMINQUEUE_TYPE_NAME},
		{"com.ibm.mqe.MQeRemoteQueue",			 	REMOTEQUEUE_TYPE_NAME},
		{"com.ibm.mqe.mqbridge.MQeMQBridgeQueue", 	REMOTEQUEUE_TYPE_NAME},
		{"com.ibm.mqe.MQeStoreAndForwardQueue", 	STOREANDFORWARDQUEUE_TYPE_NAME},
		{"com.ibm.mqe.MQeHomeServerQueue",	 		HOMESERVERQUEUE_TYPE_NAME},
		{"com.ibm.mqe.MQeQueue", 					LOCALQUEUE_TYPE_NAME},
	};

	
	private String state;
	private MQeQueueManagerStub qmStub;
	private MQeQueueManager adminQm;
	private boolean isDirect = false;
/**
 * MQeQueueManager constructor comment.
 * @param name java.lang.String
 * @param metaObject com.ibm.uia.metamodel.MetaObject
 */
public QueueManager() {
	super();
	state = STATE_DISCONNECTED;
}
/**
 * performOperation method comment.
 */
private boolean _createDirectConnection(Renderer renderer, String[] data) {
	boolean reply = false;
	String targetQueueManagerName = data[0];  // name of owning qm
	if (null != targetQueueManagerName) {
		if (null == findExistingChild(CONNECTION_TYPE_NAME, targetQueueManagerName)) {
			ManagedObject cp = (ManagedObject)addNewChild(CONNECTION_TYPE_NAME, targetQueueManagerName);
			String address = null;
			if (data[1].length() > 0 && data[2].length() > 0 && data[3].length() > 0) {
				address = data[1]+":"+data[2]+":"+data[3];
			} else if (data[2].length() > 0) {
				address = data[2];
			}
			cp.setAttribute(CONNECTION_ROUTE_ATTRIBUTE, address);
			cp.setAttribute(DEST_QM_NAME_ATTRIBUTE, targetQueueManagerName);
			cp.create();
			reply = true;
		}
	}
	return reply;
}
/**
 * performOperation method comment.
 */
private boolean _createHomeServerQueue(Renderer renderer, String[] data) {
	boolean reply = false;
	String queueName = data[0];
	String targetQueueManagerName = data[1];  // name of owning qm
	if (null != queueName) {
		if (null == findExistingChild(HOMESERVERQUEUE_TYPE_NAME, queueName)) {
			ManagedObject qp = (ManagedObject)addNewChild(HOMESERVERQUEUE_TYPE_NAME, queueName);
			qp.setAttribute(QQMNAME_ATTR, targetQueueManagerName);
			reply = true;
			qp.create();
		}
	}
	return reply;
}
/**
 * performOperation method comment.
 */
private boolean _createIndirectConnection(Renderer renderer, String[] data) {
	boolean reply = false;
	String targetQueueManagerName = data[0];  // name of owning qm
	if (null != targetQueueManagerName) {
		if (null == findExistingChild(CONNECTION_TYPE_NAME, targetQueueManagerName)) {
			ManagedObject cp = (ManagedObject)addNewChild(CONNECTION_TYPE_NAME, targetQueueManagerName);
			String address = null;
			if (data[1].length() > 0) {
				address = data[1];
			}
			cp.setAttribute(CONNECTION_ROUTE_ATTRIBUTE, address);
			cp.setAttribute(DEST_QM_NAME_ATTRIBUTE, targetQueueManagerName);
			cp.create();
			reply = true;
		}
	}
	return reply;
}
/**
 * performOperation method comment.
 */
private boolean _createLocalQueue(Renderer renderer, String[] data) {
	boolean reply = false;
	if (null != data) {
		String queueName = data[0];
		if (null == findExistingChild(LOCALQUEUE_TYPE_NAME, queueName)) {
			ManagedObject qp = (ManagedObject)addNewChild(LOCALQUEUE_TYPE_NAME, queueName);
			reply = true;
			qp.create();
		}
	}
	return reply;
}
/**
 * performOperation method comment.
 */
private boolean _createRemoteQueue(Renderer renderer, String[] data, boolean synchronous) {
	boolean reply = false;
	String queueName = data[0];
	String targetQueueManagerName = data[1];  // name of owning qm
	if (null != queueName && null != targetQueueManagerName) {
		if (null == findExistingChild(REMOTEQUEUE_TYPE_NAME, queueName)) {
			ManagedObject qp = (ManagedObject)addNewChild(REMOTEQUEUE_TYPE_NAME, queueName);
			qp.setAttribute(QQMNAME_ATTR, targetQueueManagerName);
			String mode = synchronous ? QUEUE_MODE_SYNC : QUEUE_MODE_ASYNC;
			qp.setAttribute(QUEUE_MODE_ATTRIBUTE, mode);
			reply = true;
			qp.create();
		}
	}
	return reply;
}
/**
 * performOperation method comment.
 */
private boolean _createStoreAndForwardQueue(Renderer renderer, String[] data) {
	boolean reply = false;
	String queueName = data[0];
	String targetQueueManagerName = data[1];  // name of owning qm
	if (null != queueName) {
		if (null == findExistingChild(STOREANDFORWARDQUEUE_TYPE_NAME, queueName)) {
			ManagedObject qp = (ManagedObject)addNewChild(STOREANDFORWARDQUEUE_TYPE_NAME, queueName);
			qp.setAttribute(QQMNAME_ATTR, targetQueueManagerName);
			reply = true;
			qp.create();
		}
	}
	return reply;
}
private final boolean _readConnectionRefreshData(MQeFields data, Vector found) {
	try {
		String[] names = data.getAsciiArray(MQeQueueManagerAdminMsg.QMgr_Connections);
		for (int i = 0; i < names.length; i++) {
			String name = names[i];
			CreatableObject uio = (CreatableObject)findExistingChild(CONNECTION_TYPE_NAME, name);
			if (null == uio) {
				uio = (CreatableObject)addNewChild(CONNECTION_TYPE_NAME, name);
			}
			if (null != uio) {
				found.addElement(uio);
				if (!uio.isCreated()) {
					uio.setStateCreated();
				}
				uio.refresh();
			}
		}
	} catch (Exception e) {
		_comment("Error reading queue manager connections", e);
	}
	return true;
}
/**
 * Examine the given MQeFields object that should represent one queue.
 * If the does not exist as a hild of the receiver, then create it.
 * Return the queue found or created
 */
protected CreatableObject _readQueueRefreshData(MQeFields queueData) {
	CreatableObject queue = null;
	try {
		String queueManagerName = queueData.getAscii(MQeQueueManagerAdminMsg.QMgr_QueueQMgrName);
		String queueName = queueData.getAscii(MQeQueueManagerAdminMsg.QMgr_QueueName);
		String queueClassName = queueData.getAscii(MQeQueueManagerAdminMsg.QMgr_QueueType);
		queueClassName = MQe.abbreviate(queueClassName, 0);
		String queueType = typeForClass(queueClassName);
		queue = (CreatableObject) findExistingChild(queueType, queueName);
		if (null == queue) {
			queue = (CreatableObject) addNewChild(queueType, queueName);
		}
		if (null != queue) {
			queue.setAttribute(QQMNAME_ATTR, queueManagerName);
			if (!queue.isCreated()) {
				queue.setStateCreated();
			}
			queue.refresh();
		}
	} catch (Exception e) {
		_comment("Error reading queue manager queues", e);
	}
	return queue;
}
/**
 * The queue manager has received an MQeFields that may contain descriptions of
 * of MQeQueues.  Find or create the queues, and add them to the ector of
 * found UIObjects.
 */
private final boolean _readQueueRefreshData(MQeFields data, Vector foundQs) {
	MQeFields[] queues = null;
	try {
		queues = data.getFieldsArray(MQeQueueManagerAdminMsg.QMgr_Queues);
	} catch (Exception e) {
		//_comment("No queues to read");
	}
	if (null != queues) {
		for (int i = 0; i < queues.length; i++) {
			CreatableObject uio = _readQueueRefreshData(queues[i]);
			if (null != uio) {
				foundQs.addElement(uio);
			}
		}
	}
	return true;
}
/**
 * Read the receivers data from the output data field
 */
protected boolean _readRefreshData(MQeFields data, Vector found) {
	_readQueueRefreshData(data, found);
	_readConnectionRefreshData(data, found);
	_addPendingChildrenTo(found);
	return true;
}
final boolean[] addTemporaryRoute(String destination) {
	return getQueueManagerStub().addTemporaryRoute(destination, getName());
}
/**
 * browse messages assciated with the receiver
 */
public MQeEnumeration browseQueue(String queueQueueManagerName, String queueName) {
	return getQueueManagerStub().browseQueue(queueQueueManagerName, queueName);
}
/**
 * Remove any resources associated with the receiver, 
 * as the system is closing down.
 * 
 * disconnect from the administered queue manager
 */
public void cleanUp() {
	disconnect();
}
final boolean connect(int mode) {
	setStateConnecting();
	getRenderer().redisplay(false);
	boolean connected = getQueueManagerStub().connect(getName(), getRoute(), mode);
	if (connected) {
		_setAttributeEditable(OBJECT_NAME, false);
		_setAttributeEditable(QM_ROUTE_ATTRIBUTE, false);
		setStateConnected();
	} else {
		setStateDisconnected();
	}
	return true;
}
public boolean[] createTemporaryRemoteQueue(String queueQueueManagerName, String queueName) {
	return getQueueManagerStub().createTemporaryRemoteQueue(queueQueueManagerName, queueName);
}
public boolean deleteMessage(String queueQueueManagerName, String queueName, MQeFields msg) {
	return getQueueManagerStub().deleteMessage(queueQueueManagerName, queueName, msg);
}
public void deleteTemporaryRemoteQueue(String queueQueueManagerName, String queueName) {
	getQueueManagerStub().deleteTemporaryRemoteQueue(queueQueueManagerName, queueName);
}
final boolean disconnect() {
	setStateDisconnecting();
	getRenderer().redisplay(false);
	boolean disconnected = getQueueManagerStub().disconnect(false);
	if (disconnected) {
		setStateDisconnected();
	} else {
		setStateConnected();
	}
	return true;
}
public java.awt.Image getCaptionImage() {
	Image im = null;
	if (isConnected()) {
		im = getImage("_qmanagerUp.gif");
	} else {
		im =getImage("_qmanagerDown.gif");
	}
	return im;
}
public final Color getDisplayColour() {
	if (isDisconnected()) {
		return Color.blue;
	} else if (isConnecting()) {
		return Color.cyan;
	} else if (isDisconnected()) {
		return Color.red;
	} else if (isDisconnecting()) {
		return Color.magenta;
	}
	return Color.black;
}
public String getHintText() {
	return getType() + ": " + getName() + " (" + state + ")";
}
public final QueueManager getQueueManager() {
	return this;
}
public final MQeQueueManagerStub getQueueManagerStub() {
	return qmStub;
}
final String getRoute() {
	return getAttributeValue(QM_ROUTE_ATTRIBUTE);
}
public final boolean isConnected() {
	if( getQueueManagerStub().isConnected()) {
		setStateConnected();
		return true;
	} else {
		setStateDisconnected();
		return false;
	}
}
public final boolean isConnecting() {
	return STATE_CONNECTING == state;
}
public final boolean isDisconnected() {
	return STATE_DISCONNECTED == state;
}
public final boolean isDisconnecting() {
	return STATE_DISCONNECTING == state;
}
/**
 * Reply true if the supplied renderable operation is currently enabled 
 * for the receiver.  Reply false if the operation is not currently
 * enabled.
 */
public boolean isOperationEnabled(RenderableOperation rop) {
	boolean reply = true;
	String cmd = rop.getName();
	if (CMD_COMMIT.equals(cmd)) {
		reply = isConnected();
	} else if (CMD_CONNECT_ASYNC.equals(cmd)) {
		reply = !isConnected() && !isDirect;
	} else if (CMD_CONNECT_SYNC.equals(cmd)) {
		reply = !isConnected() && !isDirect;
	} else if (CMD_DISCONNECT.equals(cmd)) {
		reply = isConnected() && !isDirect;
	} else {
		reply = super.isOperationEnabled(rop);
	}
	return reply;
}
/**
 * Perform the actions represented by the supplied instance of 
 * <code>RenderableOperation</code> defined in te metamodel file.
 * If the operation was defined with a prompt then the String array
 * will be non-null, and will contain a string for every line of the prompt
 *
 * The renderer responsible for delivering the operation request is passed in
 * for convenience.
 *
 * Reply true if a stuctural refresh is required.
 */
public boolean performOperation(RenderableOperation op, Renderer renderer) {
	String[] data = op.getData();
	String command = op.getName();
	boolean reply = false;
	if (CMD_CONNECT_ASYNC.equals(command)) {
		//_comment("connect async command start");
		connect(getQueueManagerStub().CONNECTION_MODE_ASYNCHRONOUS);
		//_comment("connect command end");
	} else if (CMD_CONNECT_SYNC.equals(command)) {
		//_comment("connect command start");
		connect(getQueueManagerStub().CONNECTION_MODE_SYNCHRONOUS);
		//_comment("connect command end");
	} else if (CMD_DISCONNECT.equals(command)) {
		disconnect();
	} else if (CMD_REFRESH.equals(command)) {
		//_comment("refresh command start");
		reply = refresh();
		//_comment("refresh command end");
	} else if (CMD_COMMIT.equals(command)) {
		reply = commit(true);
	} else if (CMD_UNDO_CHANGES.equals(command)) {
		undo(true);
		reply = true;
	} else if (CMD_NEW_LOCAL_QUEUE.equals(command)) {
		reply = _createLocalQueue(renderer, data);
	} else if (CMD_NEW_REMOTE_QUEUE_SYNC.equals(command)) {
		reply = _createRemoteQueue(renderer, data, true);
	} else if (CMD_NEW_REMOTE_QUEUE_ASYNC.equals(command)) {
		reply = _createRemoteQueue(renderer, data, false);
	} else if (CMD_NEW_STOREANDFORWARD_QUEUE.equals(command)) {
		reply = _createStoreAndForwardQueue(renderer, data);
	} else if (CMD_NEW_HOMESERVER_QUEUE.equals(command)) {
		reply = _createHomeServerQueue(renderer, data);
	} else if (CMD_NEW_CONNECTION.equals(command)) {
		reply = _createDirectConnection(renderer, data);
	} else if (CMD_NEW_INDIRECT_CONNECTION.equals(command)) {
		reply = _createIndirectConnection(renderer, data);
	} else {
		reply = super.performOperation(op, renderer);
	}
	return reply;
}
/**
 * put an admin message.
 *
 * If no listener has been set then the method waits for a response to 
 * the admin message, and returns it.
 * If a listener has been set, then the message will be returned
 * asynchronously via the listeners acceptMessage() method.
 *
 * This implementation of the method adds the queueManager specific
 * data to the request and passes it to the administrating queueManager.
 */
public final MQeAdminMsg putAdminMessage(MQeAdminMsg msg, MQeStubMessageListener listener) {
	return qmStub.putAdminMessage(msg, listener);
}
/**
 * Initiate the sequence of actions which will result in refreshing the
 * receivers data.
 * Reply true if this results in a structural change.
 */
public boolean refresh() {
	boolean reply = false;
	if (isConnected()) {
		MQeAdminMsg msg = getQueueManagerStub().inquireAll();
		reply = handleMessage(msg);
	
	}
	return reply;
}
public final void removeTemporaryRoute(String destination) {
	getQueueManagerStub().removeTemporaryRoute(destination);
}
final void setDirect(boolean direct, MQeQueueManager qm) {
	isDirect = direct;  
	setStateConnected();
	if (direct) {
		qmStub = new DirectQMStub(this, qm);
	} else {
		qmStub = new IndirectQueueManagerStub(this, qm);
	}
	qmStub.setName(getName());
}
private final void setStateConnected() {
	state = STATE_CONNECTED;
}
private final void setStateConnecting() {
	state = STATE_CONNECTING;
}
private final void setStateDisconnected() {
	state = STATE_DISCONNECTED;
}
private final void setStateDisconnecting() {
	state = STATE_DISCONNECTING;
}
/**
 * browse messages assciated with the receiver
 */
public void testMessage(String queueQueueManagerName, String queueName, String txt) {
	getQueueManagerStub().testMessage(queueQueueManagerName, queueName, txt);
}
/**
 * Reply the string representing the type as given by the
 * supplied class name.
 */
protected String typeForClass(String className) {
	for (int i = 0; i < classNameToTypeMap.length; i++) {
		if (className.equals(classNameToTypeMap[i][0])) {
			return classNameToTypeMap[i][1];
		}
	}
	return LOCALQUEUE_TYPE_NAME;  // assume local queue
}
/**
 * Initiate the sequence of actions which will result in updating the
 * resource represented by the receiver with the values curently held by
 * the receiver.
 * Reply true if this results in a structural change.
 */
protected boolean update(boolean recurse) {
	boolean reply = false;
	MQeFields sendData = new MQeFields();
	_writeAttributes(sendData);
	MQeAdminMsg msg = getQueueManagerStub().update(sendData);
	reply = handleMessage(msg);
	if (recurse) {
		Enumeration enum = getAllChildren();
		while (enum.hasMoreElements()) {
			MQeObject mqeo = (MQeObject) enum.nextElement();
			if (mqeo.update(recurse)) {
				reply = true;
			}
		}
	}
	return reply;
}
}
