package com.ibm.uia.metamodel;

/****************************************************************************/
/*                                                                          */
/* Program name: MQ Series Everyplace - Administration Tool                 */
/*                                                                          */
/*  Statement:  Licensed Materials - Property of IBM                        */
/*                                                                          */
/*              MQSeries SupportPac MSE1                                    */
/*              (C) Copyright IBM Corp. 2000                                */
/*                                                                          */
/****************************************************************************/
/*                                                                          */
/* Change history:                                                          */
/*                                                                          */
/* V1.0   15-08-2000  PTvL  Initial release                                 */
/*                                                                          */
/*==========================================================================*/
import com.ibm.uia.xml.*;
import com.ibm.uia.renderingInterface.*;
/**
 * This type defines the interface expected of an MQAttribute.
 * MQAttributes are associated with MQObjects, and represent a
 * mapping from a semantically distinguished field to the 
 * value allocated to that field.  
 * The allocated value is constrained to the valuespace defined by
 * an instance of type 
 *
 * Properly speaking this is part of the MODEL rather than the metamodel.
 */
public class Attribute extends ModelComponent implements RenderableAttribute {
	
	protected static final String ATTRIBUTE_TAG = "attribute";
	protected static final String ATTRIBUTE_NAME = "name";
	protected static final String ATTRIBUTE_VALUE = "value";
	private String value = null;
	private AttributeDefinition definition;
	private String committedValue = null;
	private boolean editable;
	private UIObject owner;
/**
 * Attribute constructor comment.
 * NOTE: we remove the attributes we use, so that
 * subclass implementations can ignore them
 */
protected Attribute(AttributeDefinition def, UIObject owner) {
	super();
	definition = def;
	editable = definition._isEditable();
	this.owner = owner;
}
public final void _comment(String s) {
	owner._comment(s);
}
public final void _comment(String s, Exception e) {
	_comment(s + " : " + e.getMessage());
}
/**
 * declare the receivers internal value to be the 
 * committed value.
 */
public final void commit() {
	committedValue = value;
}
/**
 * create a DOM representation of the receiver using
 * the supplied document.  The Element generated should
 * be returned to the caller, and not appended to the 
 * document.  Subclasses can override this method but
 * are strongly recommended to call the superclass
 * method FIRST and decorate the returned element with their
 * specific data.
 * Note that the superclass is responsible for navigating
 * the standard tree structures, so this responsibility
 * is removed from subclass implementors
 */
protected Element generateDom(Document doc) {
	Element el = super.generateDom(doc);
	el.setAttribute(ATTRIBUTE_NAME, getName());
	el.setAttribute(ATTRIBUTE_VALUE, getValue());
	return el;
}
/**
 * setValue method comment.
 */
public final String getCommittedValue() {
	if (null == committedValue) {
		committedValue = definition._getDefaultValue();
	}
	return committedValue;
}
/**
 * getDefaultValue method comment.
 */
public String getDefaultValue() {
	return definition._getDefaultValue();
}
/**
 * getDefaultValue method comment.
 */
public AttributeDefinition getDefinition() {
	return definition;
}
public java.awt.Color getDisplayColour() {
	return java.awt.Color.black;
}
public final MetaModel getMetaModel() {
	return definition.getMetaModel();
}
public final MetaObject getMetaObject() {
	return definition.getMetaObject();
}
/**
 * getName method comment.
 */
public final String getName() {
	return definition.getName();
}
/**
 * Reply the string used to identify an XML tag as representing
 * an instance of the receivers class
 */
public final String getTagName() {
	return ATTRIBUTE_TAG;
}
/**
 * getType method comment.
 */
public final RenderableAttributeType getType() {
	return definition.getType();
}
/**
 * getValue method comment.
 */
public final String getValue() {
	if (null == value) {
		value = definition._getDefaultValue();
	}
	return value;
}
public final boolean isCommitted() {
	if (null == committedValue) {
		return null == value;
	} else {
		return committedValue.equals(value);
	}
}
/**
 * isEditable method comment.
 */
public final boolean isEditable() {
	return editable;
}
/**
 * isNameAttribute method comment.
 */
public final boolean isNameAttribute() {
	return definition._isNameAttribute();
}
/**
 * isEditable method comment.
 */
public final boolean isRequired() {
	return definition._isRequired();
}
/**
 * Initialise the receiver from the given Element.
 */ 
protected void parseDom(Element element) {
	super.parseDom(element);
	String val = element.getAttribute(ATTRIBUTE_VALUE);
	element.removeAttribute(ATTRIBUTE_VALUE);
	setValue(val);
}
/**
 * reset the receivers internal value to the committed
 * value.
 */
public final void revert() {
	value = committedValue;
}
/**
 * setValue method comment.
 */
public final void setCommittedValue(String newValue) {
	committedValue = newValue;
}
/**
 * isEditable method comment.
 */
public final void setEditable(boolean ed) {
	editable = ed;
}
/**
 * setValue method comment.
 */
public void setValue(String newValue) {
	value = newValue;
}
}
