package com.ibm.uia.metamodel;

/****************************************************************************/
/*                                                                          */
/* Program name: MQ Series Everyplace - Administration Tool                 */
/*                                                                          */
/*  Statement:  Licensed Materials - Property of IBM                        */
/*                                                                          */
/*              MQSeries SupportPac MSE1                                    */
/*              (C) Copyright IBM Corp. 2000                                */
/*                                                                          */
/****************************************************************************/
/*                                                                          */
/* Change history:                                                          */
/*                                                                          */
/* V1.0   15-08-2000  PTvL  Initial release                                 */
/*                                                                          */
/*==========================================================================*/
import com.ibm.uia.renderingInterface.*;
import com.ibm.uia.xml.*;
import java.util.*;
import java.awt.*;
import java.io.*;
import java.util.zip.*;
/**
 * This is a convenience superclass for all metamodel
 * classes.
 */
public abstract class UIAComponent {
	private static final String CAPTION_TAG = "caption";
	private static final String IMAGE_TAG = "image";
	private static final String HINT_TAG = "hint";
	private static final String HELP_TAG = "help";
	public static final String NAME_TAG = "name";
		
	protected static final Vector EMPTY_VECTOR = new Vector();
	protected static final Enumeration EMPTY_ENUM = EMPTY_VECTOR.elements();

	private String name;
	private String imageName;
 	private Image captionImage;
	private String caption;
	private static Hashtable imageCache;
	private static String resourceFileName = "images.jar";
/**
 * UIAComponent constructor comment.
 */
public UIAComponent() {
	super();
}
/**
 * Remove any resources associated with the receiver, 
 * as the system is closing down.  This function is 
 * provided as part of the interface.  It as no specific
 * code here, and is designed to be overidden
 */
public void cleanUp() {
}
public String fudge() {
	Document doc = new Document();
	doc.appendChild(generateDom(doc));
	return Unparser.ToString(doc);
}
/**
 * create a DOM representation of the receiver using
 * the supplied document.  The Element generated should
 * be returned to the caller, and not appended to the 
 * document.  Subclasses can override this method but
 * are strongly recommended to call the superclass
 * method FIRST and decorate the returned element with their
 * specific data.
 * Note that the superclass is responsible for navigating
 * the standard tree structures, so this responsibility
 * is removed from subclass implementors
 */
protected Element generateDom(Document doc) {
	Element el = doc.createElement(getTagName());
	el.setAttribute(NAME_TAG, getName());
	el.setAttribute(CAPTION_TAG, getCaptionText());
	el.setAttribute(IMAGE_TAG, imageName);
	return el;
}
/**
 * reply the assistance designated 'Caption' for the receiver.
 */
public java.awt.Image getCaptionImage() {
	if (null == captionImage) {
		if (null != imageName) {
			// is it in the cache?
			captionImage = getImage(imageName);
		}
	}
	return captionImage;
}
/**
 * reply the assistance designated 'Caption' for the receiver.
 */
public String getCaptionText() {
	if (null == caption && null != name) {
		caption = name;
	}
	return caption;
}
/**
 * reply the assistance designated 'Caption' for the receiver.
 */
public String getHelpText() {
	return "";
}
/**
 * reply the assistance designated 'Caption' for the receiver.
 */
public String getHintText() {
	return "";
}
protected final static Image getImage(String imageName) {
	Image image = (Image) getImageCache().get(imageName);
	if (null == image) {
		try {
			ZipFile zipFile = new ZipFile(resourceFileName);
			ZipEntry zipEntry = zipFile.getEntry(imageName);
			if (null != zipEntry) {
				int maxRead = 10000;
				byte[] data = new byte[maxRead];
				int bytesRead = (int)zipFile.getInputStream(zipEntry).read(data, 0, maxRead);
				image = Toolkit.getDefaultToolkit().createImage(data, 0, (int)bytesRead);
			}
		} catch (Exception e) {
			System.out.println("failed to read " + imageName + " from " + resourceFileName);
		}
		if (null == image) {
			// try to find file on disc
			image = Toolkit.getDefaultToolkit().getImage(imageName);
		}
		imageCache.put(imageName, image);
	}
	return image;
}
private static final Hashtable getImageCache() {
	if (null == imageCache) {
		imageCache = new Hashtable();
		/*
		try {
			Toolkit toolkit = Toolkit.getDefaultToolkit();
			ZipInputStream zipInputStream = new ZipInputStream(
				new BufferedInputStream(
					new FileInputStream(resourceFileName)));
			//System.out.println("opened zip file");
			ZipEntry zipEntry = zipInputStream.getNextEntry();
			while (null != zipEntry) {
				int maxRead = 10000;
				byte[] data = new byte[maxRead];
				String entryName = zipEntry.getName();
				if (entryName.endsWith(".bmp") || entryName.endsWith(".gif")) {
					//System.out.println("Reading entry: " + entryName);
					long bytesRead = zipInputStream.read(data, 0, maxRead);
					//System.out.println("bytes read = " + bytesRead);
					Image image = toolkit.createImage(data, 0, (int)bytesRead);
					//System.out.println("created image");
					imageCache.put(entryName, image);
				}
				zipEntry = zipInputStream.getNextEntry();
			}
		} catch (IOException e) {
			System.out.println("Error reading images file: " + e.getMessage());
			
		}
		*/
		
	}
	return imageCache;
}
protected abstract MetaModel getMetaModel();
public String getName() {
	if (null == name && null != caption) {
		name = caption;
	}
	return name;
}
protected String getSortName() {
	return getName();
}
/**
 * Reply the string used to identify an XML tag as representing
 * an instance of the receivers class
 */
public abstract String getTagName();
/**
 * Initialise the receiver from the given Element.
 */ 
public void initialiseFrom(Element element) {
	parseDom(element);
}
/**
 * Initialise a child of the receiver from the given Element.
 */ 
protected UIAComponent parseChildElement(Element element) {
	String elemName = element.getName();
	if (CAPTION_TAG.equals(elemName)) {
		//caption.parseDom(element);
	} else if (HELP_TAG.equals(elemName)) {
		//help.parseDom(element);
	} else if (HINT_TAG.equals(elemName)) {
		//hint.parseDom(element);
	}
	return null;
}
/**
 * Initialise the receiver from the given Element.
 */ 
protected void parseDom(Element element) {
	name = element.getAttribute(NAME_TAG);
	caption = element.getAttribute(CAPTION_TAG);
	imageName = element.getAttribute(IMAGE_TAG);
	Enumeration enum = element.getChildren();
	while (enum.hasMoreElements()) {
		Element child = (Element)enum.nextElement();
		parseChildElement(child);
	}
}
/**
 * reply the assistance designated 'Caption' for the receiver.
 */
public void setCaptionText(String newCaption) {
	caption = newCaption;
}
public final void setName(String newName) {
	name = newName;
}
protected static final void setResourceFileName(String fileName) {
	resourceFileName = fileName;
}
public static final Enumeration sorted(Enumeration enum) {
	Vector v = new Vector();
	if (enum.hasMoreElements()) {
		v.addElement(enum.nextElement());
	}
	while (enum.hasMoreElements()) {
		UIAComponent newComp = (UIAComponent)enum.nextElement();
		String newName = newComp.getSortName();
		boolean inserted = false;
		for (int i = 0; i < v.size() && !inserted; i++) {
			UIAComponent oldComp = (UIAComponent)v.elementAt(i);
			if (oldComp.getSortName().compareTo(newName) >= 0) {
				v.insertElementAt(newComp, i);
				inserted = true;
			}
		}
		if (!inserted) {
			v.addElement(newComp);
		}
	}
	return v.elements();
}
public static final Enumeration sortedString(Enumeration enum) {
	Vector v = new Vector();
	if (enum.hasMoreElements()) {
		v.addElement(enum.nextElement());
	}
	while (enum.hasMoreElements()) {
		String newName = enum.nextElement().toString();
		boolean inserted = false;
		for (int i = 0; i < v.size() && !inserted; i++) {
			String oldName = v.elementAt(i).toString();
			if (oldName.compareTo(newName) >= 0) {
				v.insertElementAt(newName, i);
				inserted = true;
			}
		}
		if (!inserted) {
			v.addElement(newName);
		}
	}
	return v.elements();
}
public String toString() {
	Document doc = new Document();
	doc.appendChild(generateDom(doc));
	return Unparser.ToString(doc);
}
}
