package com.ibm.uia.renderer;

/****************************************************************************/
/*                                                                          */
/* Program name: MQ Series Everyplace - Administration Tool                 */
/*                                                                          */
/*  Statement:  Licensed Materials - Property of IBM                        */
/*                                                                          */
/*              MQSeries SupportPac MSE1                                    */
/*              (C) Copyright IBM Corp. 2000                                */
/*                                                                          */
/****************************************************************************/
/*                                                                          */
/* Change history:                                                          */
/*                                                                          */
/* V1.0   15-08-2000  PTvL  Initial release                                 */
/*                                                                          */
/*==========================================================================*/
import com.ibm.uia.renderingInterface.*;
import java.awt.image.*;
import java.awt.*;
import java.awt.event.*;
import java.util.*;

/**
 * This type was created in VisualAge.
 */
public abstract class ComponentPanel extends Panel 
		implements MouseListener, MouseMotionListener, ImageObserver, KeyListener {
	protected RendererPanel rendererPanel;
	RendererNode selectedNode;
	protected Vector rootNodes;
	protected int maxWidth;
	protected int height;
/**
 * RenderPanel constructor comment.
 */
public ComponentPanel(RendererPanel renderer) {
	super();
	rendererPanel = renderer;
	rootNodes = new Vector();
	addMouseListener(this);
	addMouseMotionListener(this);
	addKeyListener(this);
}
protected final void _comment(String s) {
	//System.out.println(s);
}
protected void _doubleClicked(RendererNode rn) {
}
protected final RendererNode _getNode(Point pt) {
	Enumeration enum = rootNodes.elements();
	while (enum.hasMoreElements()) {
		RendererNode rtn = (RendererNode)enum.nextElement();
		rtn = rtn.find(pt);
		if (null != rtn) {
			return rtn;
		}
	}
	return null;
}
final RendererNode _getSelectedNode() {
	return selectedNode;
}
boolean _setSelectedNode(RendererNode select) {
	if (null != selectedNode) {
		//System.out.println("deselecting " + selectedNode._getText());
		selectedNode.select(false);
	}
	selectedNode = select;
	if (null != selectedNode) {
		//System.out.println("selecting " + selectedNode._getText());
		selectedNode.select(true);
	}
	return true;
}
/** 
 * Gets the preferred size of this component.
 * @return A dimension object indicating this component's preferred size.
 * @see #getMinimumSize
 * @see java.awt.LayoutManager
 */
public Dimension getPreferredSize() {
	Dimension d = super.getPreferredSize();
	if (d.height < height) {
		d.height = height;
	}
	if (d.width < maxWidth) {
		d.width = maxWidth;
	}
	return d;
}
final Renderer getRenderer() {
	return rendererPanel.getRenderer();
}
public final Enumeration getRootObjects() {
	return rendererPanel.getRootObjects();
}
/**
 * This method is called when information about an image which was
 * previously requested using an asynchronous interface becomes
 * available.  Asynchronous interfaces are method calls such as
 * getWidth(ImageObserver) and drawImage(img, x, y, ImageObserver)
 * which take an ImageObserver object as an argument.  Those methods
 * register the caller as interested either in information about
 * the overall image itself (in the case of getWidth(ImageObserver))
 * or about an output version of an image (in the case of the
 * drawImage(img, x, y, [w, h,] ImageObserver) call).  

 * <p>This method
 * should return true if further updates are needed or false if the
 * required information has been acquired.  The image which was being
 * tracked is passed in using the img argument.  Various constants
 * are combined to form the infoflags argument which indicates what
 * information about the image is now available.  The interpretation
 * of the x, y, width, and height arguments depends on the contents
 * of the infoflags argument.
 * @see Image#getWidth
 * @see Image#getHeight
 * @see java.awt.Graphics#drawImage
 */
public boolean imageUpdate(Image img, int infoflags, int x, int y, int width, int height) {
	if ((infoflags & ALLBITS) == ALLBITS) {
		_comment("Image update " + infoflags);
		repaint();
	}
	return true;
}
protected boolean isPopupTrigger(MouseEvent e) {
	// isPopupTrigger doesnt seem robust, check for button mask
	if (rendererPanel.usePopupMenu()) {
		return ((e.getModifiers() & e.BUTTON3_MASK) == e.BUTTON3_MASK)
				||	(e.isPopupTrigger());
	}
	return false;

}
public void keyPressed(KeyEvent e) {
	int key = e.getKeyCode();
	int mod = e.getModifiers();
	switch (key) {
		case e.VK_C:	//copy??
			if (e.CTRL_MASK == mod) {
				RendererNode ro = _getSelectedNode();
				if (null != ro) {
					if (ro.copy()) {
					}
				}
			}
			break;
		case e.VK_X:	// cut??
			if (e.CTRL_MASK == mod) {
				RendererNode ro = _getSelectedNode();
				if (null != ro) {
					if (ro.cut()) {
						recalculate();
						repaint();
					}
				}
			}
			break;
		case e.VK_V:	// paste??
			if (e.CTRL_MASK == mod) {
				RendererNode ro = _getSelectedNode();
				if (null != ro) {
					if (ro.paste()) {
						recalculate();
						repaint();
					}
				}
			}
			break;
		case e.VK_UP:
			moveCursor(-1);
			break;
		case e.VK_DOWN:
			moveCursor(+1);
			break;
		case e.VK_ENTER:
			//okPressed();
			break;
		case e.VK_ESCAPE:
			//escPressed();
			break;
	}
}
public void keyReleased(KeyEvent e) {
	;
}
public void keyTyped(KeyEvent e) {
	;
}
/**
 * mouseClicked method comment.
 */
public void mouseClicked(MouseEvent e) {
	RendererNode rtn = _getNode(e.getPoint());
	_setSelectedNode(rtn);
	if (e.getClickCount() > 1) {
		_doubleClicked(rtn);
	}
}
public final void mouseDragged(MouseEvent e) {
}
/**
 * mouseEntered method comment.
 */
public void mouseEntered(java.awt.event.MouseEvent e) {
}
/**
 * mouseExited method comment.
 */
public void mouseExited(java.awt.event.MouseEvent e) {
}
public final void mouseMoved(MouseEvent e) {
	Point pt = e.getPoint();
	RendererNode rtn = _getNode(pt);
	rendererPanel.mouseOver(rtn);
}
/**
 * mousePressed method comment.
 */
public void mousePressed(MouseEvent e) {
	Point pt = e.getPoint();
	RendererNode rtn = _getNode(pt);
	_setSelectedNode(rtn);
	if (isPopupTrigger(e)) {
		if (null != rtn) {
			PopupMenu popup = rtn.getPopupMenu(getRenderer());
			if (null != popup) {
				this.add(popup);
				popup.show(this, pt.x, pt.y);
			}
		}
	}
	repaint();
}
/**
 * mouseReleased method comment.
 */
public void mouseReleased(java.awt.event.MouseEvent e) {
}
void moveCursor(int moveBy) {
	Vector visible = new Vector();
	Enumeration enum = rootNodes.elements();
	while (enum.hasMoreElements()) {
		RendererNode rn = (RendererNode)enum.nextElement();
		rn.addVisibleChildrenTo(visible);
	}
	int pos = visible.indexOf(selectedNode) + moveBy;
	selectedNode.select(false);
	if (pos < 0) {
		pos = 0;
	} else if (pos >= visible.size()) {
		pos = visible.size() - 1;
	}
	RendererNode newSelection = (RendererNode)visible.elementAt(pos);
	if (_setSelectedNode(newSelection)) {
		repaint();
	}
}
/** 
 * Paints the container.  This forwards the paint to any lightweight components 
 * that are children of this container.  If this method is reimplemented, 
 * super.paint(g) should be called so that lightweight components are properly
 * rendered.  If a child component is entirely clipped by the current clipping
 * setting in g, paint() will not be forwarded to that child.
 *
 * @param g the specified Graphics window
 * @see   java.awt.Component#update(java.awt.Graphics)
 */
public void paint(Graphics g) {
	//super.paint(g);
	FontMetrics fmet = g.getFontMetrics();
	height = 0;
	maxWidth = 0;
	Enumeration enum = rootNodes.elements();
	while (enum.hasMoreElements()) {
		RendererNode rtn = (RendererNode)enum.nextElement();
		int w = rtn.getWidth();
		if (w > maxWidth) {
			maxWidth = w;
		}
		height = rtn.draw(g, height, false, this);
	}
}
final void recalculate() {
	recalculate(true);
}
abstract void recalculate(boolean recurse);
}
