package com.ibm.uia.xml;

/****************************************************************************/
/*                                                                          */
/* Program name: MQ Series Everyplace - Administration Tool                 */
/*                                                                          */
/*  Statement:  Licensed Materials - Property of IBM                        */
/*                                                                          */
/*              MQSeries SupportPac MSE1                                    */
/*              (C) Copyright IBM Corp. 2000                                */
/*                                                                          */
/****************************************************************************/
/*                                                                          */
/* Change history:                                                          */
/*                                                                          */
/* V1.0   15-08-2000  PTvL  Initial release                                 */
/*                                                                          */
/*==========================================================================*/
import java.util.*;
import java.io.*;

/**
 * Simple XML parser based upon assumptions:
 *	-	Tags only
 *	-	No validation
 *	-	'<' and '>' only appear as XML syntax (others are &lt, &gt)
 */
public final class Parser {
	// we can push and pop tags onto a stack as we traverse the tree.
	Vector tagStack;
	private static Parser singleton;
/**
 * Parser constructor comment.
 */
public Parser() {
	super();
	tagStack = new Vector();
}
private final Element _currentTag() {
	Element tag = null;
	int size = tagStack.size();
	if (size > 0) {
		tag = (Element)tagStack.lastElement();
	}
	return tag;
}
private final Element _firstTag(Reader inReader) {
	tagStack = new Vector();
	TagStream tagStream = new TagStream(inReader);
	return tagStream.nextTag(); 
}
private final Element _parse(Reader inReader) {
	tagStack = new Vector();
	TagStream tagStream = new TagStream(inReader);
	Element tag = null;
	Element firstTag = tagStream.nextTag(); 
	if (null != firstTag) {
		_pushTag(firstTag);
		tag = tagStream.nextTag();
	}
	while (null != tag) {
		if (tag._isEmptyTag()) {
			_currentTag().addChild(tag);
		} else if (tag._isEndTag()) {
			if (tag._isEndTagFor(_currentTag())) {
				_popTag();
			//} else {
				// TODO throw an error here: 'mismatched tags'
			}
		} else {
			_currentTag().addChild(tag);
			_pushTag(tag);
		}
		//System.out.println(tag);
		tag = tagStream.nextTag();
	}
	return firstTag;
}
private final Element _popTag() {
	Element tag = null;
	int size = tagStack.size();
	if (size > 0) {
		tag = (Element)tagStack.lastElement();
		tagStack.removeElementAt(size-1);
	}
	return tag;
}
private final void _pushTag(Element tag) {
	tagStack.addElement(tag);
}
private static Parser _singleton() {
	if (null == singleton) {
		singleton = new Parser();
	}
	return singleton;
}
public final Element firstTag(Reader inReader) {
	return _firstTag(inReader);
}
public final Element firstTag(String str) {
	return _firstTag(new StringReader(str));
}
public static final Element FirstTag(String inString) {
	return _singleton().firstTag(new StringReader(inString));
}
public final Document parse(Reader inReader) {
	Document doc = new Document();
	doc.appendChild(_parse(inReader));
	return doc;
}
public static final Document Parse(Reader inReader) {
	return _singleton().parse(inReader);
}
public final Document parse(String inString) {
	return parse(new StringReader(inString));
}
public static final Document Parse(String inString) {
	return _singleton().parse(new StringReader(inString));
}
public final Document parseFile(String filename) {
	Document doc = null;
	try {
		File file = new File(filename);
		FileReader fr = new FileReader(file);
		doc = parse(fr);
	} catch (IOException e) {
		System.out.println(e);
		e.printStackTrace();
	}
	return doc;
}
public static final Document ParseFile(String filename) {
	Document ftag = null;
	try {
		File file = new File(filename);
		FileReader fr = new FileReader(file);
		ftag = _singleton().parse(fr);
		fr.close();
	} catch (IOException e) {
		System.out.println(e);
		e.printStackTrace();
	}
	return ftag;
}
}
